import * as React from 'react';
import tokens from '@contentful/f36-tokens';
import { FieldConnector } from '@contentful/field-editor-shared';
import { css } from 'emotion';
import { MarkdownBottomBar, MarkdownHelp } from './components/MarkdownBottomBar';
import { MarkdownConstraints } from './components/MarkdownConstraints';
import MarkdownPreviewSkeleton from './components/MarkdownPreviewSkeleton';
import { MarkdownTabs } from './components/MarkdownTabs';
import { MarkdownTextarea } from './components/MarkdownTextarea/MarkdownTextarea';
import { MarkdownToolbar } from './components/MarkdownToolbar';
import { openCheatsheetModal } from './dialogs/CheatsheetModalDialog';
import { createMarkdownActions } from './MarkdownActions';
const MarkdownPreview = /*#__PURE__*/ React.lazy(()=>import('./components/MarkdownPreview'));
const styles = {
    container: css({
        display: 'flex',
        flexDirection: 'column',
        fontFamily: tokens.fontStackPrimary
    })
};
export function MarkdownEditor(props) {
    const prevExternalReset = React.useRef(props.externalReset);
    const [currentValue, setCurrentValue] = React.useState(props.value ?? '');
    const [selectedTab, setSelectedTab] = React.useState('editor');
    const [editor, setEditor] = React.useState(null);
    const [canUploadAssets, setCanUploadAssets] = React.useState(false);
    React.useEffect(()=>{
        if (props.enableTab) {
            setSelectedTab(props.enableTab);
        }
    }, [
        props.enableTab
    ]);
    React.useEffect(()=>{
        if (editor && props.onReady) {
            props.onReady(editor);
            setTimeout(()=>{
                editor.refresh();
            }, 1);
        }
    }, [
        editor
    ]);
    React.useEffect(()=>{
        props.sdk.access.can('create', 'Asset').then((value)=>{
            setCanUploadAssets(value);
        });
    }, []);
    React.useEffect(()=>{
        if (editor) {
            editor.setReadOnly(!!props.isDisabled);
        }
    }, [
        editor,
        props.isDisabled
    ]);
    React.useEffect(()=>{
        if (props.externalReset !== prevExternalReset.current) {
            prevExternalReset.current = props.externalReset;
            editor?.setContent(props.value ?? '');
        }
    }, [
        props.value,
        props.externalReset,
        editor
    ]);
    const isActionDisabled = editor === null || props.isDisabled || selectedTab !== 'editor';
    const direction = props.sdk.locales.direction[props.sdk.field.locale] ?? 'ltr';
    const actions = React.useMemo(()=>{
        return createMarkdownActions({
            sdk: props.sdk,
            editor,
            locale: props.sdk.field.locale
        });
    }, [
        editor
    ]);
    const openMarkdownHelp = React.useCallback(()=>{
        openCheatsheetModal(props.sdk.dialogs);
    }, []);
    return /*#__PURE__*/ React.createElement("div", {
        className: styles.container,
        "data-test-id": "markdown-editor"
    }, /*#__PURE__*/ React.createElement(MarkdownTabs, {
        active: selectedTab,
        onSelect: (tab)=>{
            if (props.enableTab) return;
            setSelectedTab(tab);
        },
        enableTab: props.enableTab
    }), /*#__PURE__*/ React.createElement(MarkdownToolbar, {
        mode: "default",
        disabled: isActionDisabled,
        canUploadAssets: canUploadAssets,
        actions: actions
    }), /*#__PURE__*/ React.createElement(MarkdownTextarea, {
        minHeight: props.minHeight,
        mode: "default",
        visible: selectedTab === 'editor',
        disabled: isActionDisabled,
        direction: direction,
        onReady: (editor)=>{
            editor.setContent(props.value ?? '');
            editor.setReadOnly(!!props.isDisabled);
            setEditor(editor);
            editor.events.onChange((value)=>{
                const trimmedValue = value.replace(/^\s+$/gm, '');
                props.saveValueToSDK(trimmedValue);
                setCurrentValue(value);
            });
        }
    }), selectedTab === 'preview' && /*#__PURE__*/ React.createElement(React.Suspense, {
        fallback: /*#__PURE__*/ React.createElement(MarkdownPreviewSkeleton, null)
    }, /*#__PURE__*/ React.createElement(MarkdownPreview, {
        direction: direction,
        minHeight: props.minHeight,
        mode: "default",
        value: currentValue,
        previewComponents: props.previewComponents
    })), /*#__PURE__*/ React.createElement(MarkdownBottomBar, null, /*#__PURE__*/ React.createElement(MarkdownHelp, {
        mode: selectedTab,
        onClick: openMarkdownHelp
    })), /*#__PURE__*/ React.createElement(MarkdownConstraints, {
        sdk: props.sdk,
        value: currentValue
    }));
}
export function MarkdownEditorConnected(props) {
    return /*#__PURE__*/ React.createElement(FieldConnector, {
        debounce: 300,
        field: props.sdk.field,
        isInitiallyDisabled: props.isInitiallyDisabled,
        isDisabled: props.isDisabled
    }, ({ value, disabled, setValue, externalReset })=>/*#__PURE__*/ React.createElement(MarkdownEditor, {
            ...props,
            value: value,
            isDisabled: disabled,
            saveValueToSDK: setValue,
            externalReset: externalReset
        }));
}
