import max from 'lodash/max';
import min from 'lodash/min';
import range from 'lodash/range';
import times from 'lodash/times';
import repeat from 'lodash/repeat';
function createPrefixToggleFn(prefix) {
    return (editor)=>{
        if (editor.lineStartsWith(prefix)) {
            editor.removeFromLineBeginning(prefix.length);
        } else {
            editor.insertAtLineBeginning(prefix);
        }
    };
}
function wrapSelection(editor, marker, emptyText) {
    return ()=>{
        editor.usePrimarySelection();
        if (editor.getSelection()) {
            const selectedText = editor.getSelectedText();
            if (selectedText.startsWith(marker) && selectedText.endsWith(marker)) {
                const markerLength = marker.length;
                const textWithoutMarker = selectedText.slice(markerLength, selectedText.length - markerLength);
                editor.replaceSelectedText(textWithoutMarker);
            } else {
                editor.wrapSelection(marker);
            }
        } else {
            editor.insertAtCursor(marker + emptyText + marker);
            editor.selectBackwards(marker.length, emptyText.length);
        }
    };
}
const HEADER_CHAR = '#';
const quoteToggleFn = createPrefixToggleFn('> ');
const codeToggleFn = createPrefixToggleFn('    ');
export function create(editor) {
    return {
        bold: wrapSelection(editor, '__', 'text in bold'),
        italic: wrapSelection(editor, '*', 'text in italic'),
        strike: wrapSelection(editor, '~~', 'striked out'),
        quote: modifySelection(editor, quoteToggleFn),
        code: modifySelection(editor, codeToggleFn),
        link,
        h1: toggleHeader(editor, 1),
        h2: toggleHeader(editor, 2),
        h3: toggleHeader(editor, 3),
        ul: modifySelection(editor, ulToggleFn, true),
        ol: modifySelection(editor, olToggleFn, true),
        undo: function() {
            editor.cmd('undo');
        },
        redo: function() {
            editor.cmd('redo');
        },
        hr,
        indent,
        dedent,
        table
    };
    function hr() {
        editor.moveIfNotEmpty();
        const nl = editor.getNl();
        const markup = nl + '---' + nl + nl;
        editor.insertAtCursor(markup);
    }
    function indent() {
        editor.insertAtLineBeginning(editor.getIndentation());
    }
    function dedent() {
        const indentation = editor.getIndentation();
        if (editor.lineStartsWith(indentation)) {
            editor.removeFromLineBeginning(indentation.length);
        }
    }
    function table(config) {
        const nl = editor.getNl();
        editor.moveIfNotEmpty();
        editor.insertAtCursor(nl);
        const line = editor.getCurrentLineNumber();
        editor.insertAtCursor(tableTemplate(config.rows, config.cols).join(nl));
        editor.insertAtCursor(nl + nl);
        editor.restoreCursor(2, line);
    }
    function link(url, text, title) {
        editor.usePrimarySelection();
        const linkTitle = title ? ' "' + title + '"' : '';
        const link = text ? '[' + text + '](' + url + linkTitle + ')' : '<' + url + '>';
        editor.replaceSelectedText(link, 'around');
    }
}
function modifySelection(editor, toggleFn, isList) {
    return ()=>{
        editor.usePrimarySelection();
        const selection = editor.getSelection();
        if (selection) {
            forLineIn(selection, (lineNumber, listNumber)=>{
                editor.moveToLineBeginning(lineNumber);
                toggleFn(editor, listNumber);
            });
            editor.moveToLineEnd();
        } else {
            if (isList && !getListNumber(editor) && !editor.lineStartsWith('- ')) {
                prepareListWhitespace(editor);
            }
            toggleFn(editor);
        }
    };
}
function forLineIn(selection, cb) {
    const lines = [
        selection.anchor.line,
        selection.head.line
    ];
    const maxNumber = max(lines);
    const minNumber = min(lines);
    const lineRange = range(minNumber || 0, maxNumber !== undefined ? maxNumber + 1 : undefined);
    lineRange.forEach((lineNumber, i)=>{
        cb(lineNumber, i + 1);
    });
}
function prepareListWhitespace(editor) {
    const line = editor.getCurrentLineNumber();
    const isCurrentLineEmpty = editor.isLineEmpty(line);
    const isPrevLineEmpty = line > 0 ? editor.isLineEmpty(line - 1) : false;
    const isNextLineEmpty = line < editor.getLinesCount() - 1 ? editor.isLineEmpty(line + 1) : true;
    let linesToInsert = isCurrentLineEmpty ? 2 : 4;
    if (isPrevLineEmpty) {
        linesToInsert = linesToInsert - 1;
    }
    if (isNextLineEmpty) {
        linesToInsert = linesToInsert - 1;
    }
    editor.moveToLineEnd();
    editor.insertAtCursor(editor.getNl(linesToInsert));
    editor.restoreCursor(0, isCurrentLineEmpty ? line : line + 2);
}
function getListNumber(editor) {
    const result = editor.getCurrentLine().match(/^(\d+\. )/);
    return result ? result[1] : null;
}
function ulToggleFn(editor) {
    if (editor.lineStartsWith('- ')) {
        editor.removeFromLineBeginning(2);
    } else {
        const listNumber = getListNumber(editor);
        if (listNumber) {
            editor.removeFromLineBeginning(listNumber.length);
        }
        editor.insertAtLineBeginning('- ');
    }
}
function olToggleFn(editor, n) {
    const listNumber = getListNumber(editor);
    if (listNumber) {
        editor.removeFromLineBeginning(listNumber.length);
    } else {
        if (editor.lineStartsWith('- ')) {
            editor.removeFromLineBeginning(2);
        }
        editor.insertAtLineBeginning((n || 1) + '. ');
    }
}
function tableTemplate(nrows, ncols) {
    const cellWidth = new Array(11);
    const cell = ' ' + cellWidth.join(' ') + ' |';
    const separatorCell = ' ' + cellWidth.join('-') + ' |';
    let baseRow = '|';
    let separatorRow = '|';
    times(ncols, ()=>{
        baseRow += cell;
        separatorRow += separatorCell;
    });
    const bodyRows = range(nrows).map(()=>baseRow.replace(/\| {5}/g, '| Cell'));
    const headerRow = baseRow.replace(/\| {7}/g, '| Header');
    return [
        headerRow,
        separatorRow
    ].concat(bodyRows);
}
function toggleHeader(editor, level) {
    return ()=>{
        const initialCh = editor.getCurrentCharacter();
        const currentHeader = selectHeader(editor);
        const prefix = repeat(HEADER_CHAR, level);
        if (!currentHeader) {
            editor.moveToLineBeginning();
            editor.insertAtCursor(prefix + ' ');
            editor.restoreCursor(initialCh + prefix.length + 1);
            return;
        }
        if (editor.getSelectedText() === prefix) {
            editor.extendSelectionBy(1);
            const removedCh = editor.getSelectionLength();
            editor.removeSelectedText();
            editor.restoreCursor(initialCh - removedCh);
            return;
        }
        const diff = prefix.length - editor.getSelectionLength();
        editor.replaceSelectedText(prefix);
        editor.restoreCursor(initialCh + diff);
    };
}
function selectHeader(editor) {
    const result = editor.getCurrentLine().match(/^( {0,3})(#{1,6}) /);
    if (!result) {
        return null;
    }
    const indentation = result[1];
    const header = result[2];
    editor.select(getPos(0), getPos(header.length));
    return editor.getSelection();
    function getPos(modifier) {
        return {
            line: editor.getCurrentLineNumber(),
            ch: indentation.length + modifier
        };
    }
}
