import * as React from 'react';
import { Grid } from '@contentful/f36-components';
import { ChevronLeftIcon, ChevronRightIcon } from '@contentful/f36-icons';
import tokens from '@contentful/f36-tokens';
import { css, cx } from 'emotion';
import { MarkdownBottomBar, MarkdownHelp } from '../components/MarkdownBottomBar';
import MarkdownPreviewSkeleton from '../components/MarkdownPreviewSkeleton';
import { MarkdownTextarea } from '../components/MarkdownTextarea/MarkdownTextarea';
import { MarkdownToolbar } from '../components/MarkdownToolbar';
import { openCheatsheetModal } from '../dialogs/CheatsheetModalDialog';
import { createMarkdownActions } from '../MarkdownActions';
import { MarkdownDialogType } from '../types';
const MarkdownPreview = /*#__PURE__*/ React.lazy(()=>import('../components/MarkdownPreview'));
const styles = {
    root: css({
        display: 'grid',
        gridTemplateRows: 'min-content 1fr min-content',
        gridTemplateColumns: '1fr 1px 1fr',
        height: '85vh'
    }),
    topSplit: css({
        gridRow: '1 / 2',
        gridColumn: '1 / 4'
    }),
    bottomSplit: css({
        gridRow: '3 / 4',
        gridColumn: '1 / 4'
    }),
    editorSplit: css({
        gridRow: '2 / 3',
        gridColumn: '1 / 2',
        overflowY: 'scroll'
    }),
    editorSplitFullscreen: css({
        gridRow: '2 / 3',
        gridColumn: '1 / 4',
        overflowY: 'scroll'
    }),
    previewSplit: css({
        gridRow: '2 / 3',
        gridColumn: '3 / 4',
        overflowY: 'scroll'
    }),
    separator: css({
        gridRow: '2 / 3',
        gridColumn: '2 / 3',
        backgroundColor: tokens.gray400,
        width: '1px'
    }),
    button: css({
        cursor: 'pointer',
        zIndex: 105,
        height: '30px',
        backgroundColor: tokens.gray100,
        border: `1px solid ${tokens.gray400}`,
        padding: 0
    }),
    hideButton: css({
        gridRow: '2 / 3',
        gridColumn: '2 / 3',
        justifySelf: 'end',
        alignSelf: 'center'
    }),
    showButton: css({
        gridRow: '2 / 3',
        gridColumn: '3 / 4',
        justifySelf: 'end',
        alignSelf: 'center'
    }),
    icon: css({
        verticalAlign: 'middle'
    })
};
export const ZenModeModalDialog = (props)=>{
    const [currentValue, setCurrentValue] = React.useState(props.initialValue ?? '');
    const [showPreview, setShowPreview] = React.useState(true);
    const [editor, setEditor] = React.useState(null);
    React.useEffect(()=>{
        props.sdk?.window?.updateHeight('100%');
    }, []);
    React.useEffect(()=>{
        setTimeout(()=>{
            editor?.setFullsize();
            editor?.refresh();
        }, 150);
    }, [
        editor
    ]);
    const actions = React.useMemo(()=>{
        return createMarkdownActions({
            sdk: props.sdk,
            editor,
            locale: props.locale
        });
    }, [
        editor
    ]);
    actions.closeZenMode = ()=>{
        props.onClose({
            value: currentValue,
            cursor: editor?.getCursor()
        });
    };
    const direction = props.sdk.locales.direction[props.locale] ?? 'ltr';
    return /*#__PURE__*/ React.createElement(Grid, {
        className: styles.root,
        "data-test-id": "zen-mode-markdown-editor"
    }, /*#__PURE__*/ React.createElement(Grid.Item, {
        className: styles.topSplit
    }, /*#__PURE__*/ React.createElement(MarkdownToolbar, {
        mode: "zen",
        disabled: false,
        canUploadAssets: false,
        actions: actions
    })), /*#__PURE__*/ React.createElement(Grid.Item, {
        className: cx(styles.editorSplit, {
            [styles.editorSplitFullscreen]: showPreview === false
        })
    }, /*#__PURE__*/ React.createElement(MarkdownTextarea, {
        mode: "zen",
        visible: true,
        disabled: false,
        direction: direction,
        onReady: (editor)=>{
            editor.setContent(props.initialValue ?? '');
            editor.setReadOnly(false);
            setEditor(editor);
            editor.focus();
            editor.events.onChange((value)=>{
                setCurrentValue(value);
                props.saveValueToSDK(value);
            });
        }
    })), showPreview && /*#__PURE__*/ React.createElement(Grid.Item, {
        className: styles.previewSplit
    }, /*#__PURE__*/ React.createElement(React.Suspense, {
        fallback: /*#__PURE__*/ React.createElement(MarkdownPreviewSkeleton, null)
    }, /*#__PURE__*/ React.createElement(MarkdownPreview, {
        direction: direction,
        mode: "zen",
        value: currentValue,
        previewComponents: props.previewComponents
    }))), showPreview && /*#__PURE__*/ React.createElement(Grid.Item, {
        className: styles.separator
    }), showPreview && /*#__PURE__*/ React.createElement("button", {
        className: cx(styles.button, styles.hideButton),
        "aria-label": "Hide preview",
        onClick: ()=>{
            setShowPreview(false);
        }
    }, /*#__PURE__*/ React.createElement(ChevronRightIcon, {
        variant: "muted",
        size: "tiny",
        className: styles.icon
    })), !showPreview && /*#__PURE__*/ React.createElement("button", {
        className: cx(styles.button, styles.showButton),
        "aria-label": "Show preview",
        onClick: ()=>{
            setShowPreview(true);
        }
    }, /*#__PURE__*/ React.createElement(ChevronLeftIcon, {
        variant: "muted",
        size: "tiny",
        className: styles.icon
    })), /*#__PURE__*/ React.createElement(Grid.Item, {
        className: styles.bottomSplit
    }, /*#__PURE__*/ React.createElement(MarkdownBottomBar, null, /*#__PURE__*/ React.createElement(MarkdownHelp, {
        mode: "zen",
        onClick: ()=>{
            openCheatsheetModal(props.sdk.dialogs);
        }
    }))));
};
export const openZenMode = (dialogs, options)=>{
    return dialogs.openCurrent({
        width: 'fullWidth',
        shouldCloseOnEscapePress: false,
        shouldCloseOnOverlayClick: true,
        parameters: {
            type: MarkdownDialogType.zenMode,
            initialValue: options.initialValue,
            locale: options.locale
        }
    });
};
