import get from 'lodash/get';
import isObject from 'lodash/isObject';
function normalizeWhiteSpace(str) {
    if (str) {
        return str.trim().replace(/\s{2,}/g, ' ');
    } else {
        return str;
    }
}
function removeExtension(str) {
    return str.replace(/\.\w+$/g, '');
}
function fileNameToTitle(str) {
    return normalizeWhiteSpace(removeExtension(str).replace(/_/g, ' '));
}
export function replaceAssetDomain(fileUrl) {
    const assetDomainMap = {
        images: 'images.ctfassets.net',
        assets: 'assets.ctfassets.net',
        downloads: 'downloads.ctfassets.net',
        videos: 'videos.ctfassets.net'
    };
    return fileUrl.replace(/(images|assets|downloads|videos).contentful.com/, (_, p1)=>{
        return assetDomainMap[p1];
    });
}
function makeAssetLink(asset, { localeCode, fallbackCode, defaultLocaleCode }) {
    const localizedFile = get(asset, [
        'fields',
        'file',
        localeCode
    ]);
    const fallbackFile = fallbackCode ? get(asset, [
        'fields',
        'file',
        fallbackCode
    ]) : null;
    const defaultFile = get(asset, [
        'fields',
        'file',
        defaultLocaleCode
    ]);
    const file = localizedFile || fallbackFile || defaultFile;
    if (isObject(file) && file.url) {
        const title = get(asset, [
            'fields',
            'title',
            localeCode
        ]) || get(asset, [
            'fields',
            'title',
            fallbackCode || ''
        ]) || get(asset, [
            'fields',
            'title',
            defaultLocaleCode
        ]) || fileNameToTitle(file.fileName);
        const fileUrl = replaceAssetDomain(file.url);
        return {
            title,
            asset,
            url: fileUrl,
            isLocalized: Boolean(localizedFile),
            isFallback: Boolean(fallbackFile),
            asMarkdown: `![${title}](${fileUrl})`
        };
    } else {
        return null;
    }
}
export async function insertAssetLinks(assets, locales) {
    const otherLocales = assets.filter((asset)=>{
        return !get(asset, [
            'fields',
            'file',
            locales.localeCode
        ]);
    });
    const linksWithMeta = assets.map((asset)=>makeAssetLink(asset, locales)).filter((asset)=>asset !== null);
    if (otherLocales.length > 0) {
        const fallbackAssets = linksWithMeta.filter(({ isLocalized })=>!isLocalized).map(({ title, isFallback, asset })=>{
            const code = isFallback ? locales.fallbackCode : locales.defaultLocaleCode;
            return {
                title,
                thumbnailUrl: asset.fields.file[code].url,
                thumbnailAltText: title,
                description: isFallback ? `Fallback locale (${code})` : `Default locale (${code})`,
                asset: asset
            };
        });
        return {
            fallbacks: fallbackAssets,
            links: linksWithMeta
        };
    }
    return {
        links: linksWithMeta
    };
}
