import extend from 'lodash/extend';
import isObject from 'lodash/isObject';
import max from 'lodash/max';
import isFinite from 'lodash/isFinite';
import forEach from 'lodash/forEach';
function extractTitle(title) {
    title = title || '';
    title = title.replace(/^ *('|"|\()*/, '');
    title = title.replace(/('|"|\))* *$/, '');
    return title;
}
function head(text) {
    return text.split(' ').shift();
}
function tail(text) {
    const segments = text.split(' ');
    segments.shift();
    return segments.join(' ');
}
const PROCESSORS = {
    inline: function(match) {
        return {
            match: match[0],
            text: match[1],
            href: head(match[2]),
            title: extractTitle(tail(match[2]))
        };
    },
    ref: function(match) {
        return {
            match: match[0],
            text: match[1],
            id: match[2]
        };
    },
    label: function(match) {
        return {
            match: match[0],
            id: match[1],
            href: head(match[2]),
            title: extractTitle(tail(match[2]))
        };
    }
};
const REGEXS = {
    inline: /\[([^\r\n[\]]+)]\(([^\r\n)]+)\)/,
    ref: /\[([^\r\n[\]]+)] ?\[([^\r\n[\]]+)]/,
    label: /^ {0,3}\[([^\r\n[\]]+)]:\s+(.+)$/
};
export const findInline = makeFinder('inline');
export const findRefs = makeFinder('ref');
export const findLabels = makeFinder('label');
export function convertInlineToRef(text) {
    let id = findMaxLabelId(text);
    forEach(findInline(text), (inline)=>{
        id += 1;
        text = text.replace(inline.match, buildRef(inline, id));
        text += '\n' + buildLabel(inline, id);
    });
    return text;
}
function mergeLabels(text) {
    const byHref = {};
    const byOldId = {};
    forEach(findLabels(text), (label)=>{
        const alreadyAdded = byHref[label.href];
        const current = extend({}, label);
        if (!alreadyAdded) {
            byHref[current.href] = current;
        } else if (hasTitle(current) && !hasTitle(alreadyAdded)) {
            alreadyAdded.title = current.title;
        }
        byOldId[current.id] = alreadyAdded || current;
    });
    return {
        byHref: byHref,
        byOldId: byOldId
    };
}
export function rewriteRefs(text) {
    const merged = mergeLabels(text);
    const hrefToRefId = {};
    const labels = [];
    const rewrites = [];
    let i = 1;
    forEach(findRefs(text), (ref)=>{
        const oldLabel = merged.byOldId[ref.id];
        if (!oldLabel) {
            return;
        }
        const href = oldLabel.href;
        let newRefId = hrefToRefId[href];
        if (!newRefId) {
            hrefToRefId[href] = newRefId = i;
            i += 1;
            labels.push(extend({
                newId: newRefId
            }, oldLabel));
        }
        rewrites.push(extend({
            newId: newRefId
        }, ref));
    });
    forEach(findLabels(text), (label)=>{
        text = text.replace(label.match, '');
    });
    text = text.replace(/[\r\n\s]*$/, '');
    text += '\n\n';
    forEach(rewrites, (ref)=>{
        text = text.replace(ref.match, buildRef(ref, ref.newId));
    });
    forEach(labels, (label)=>{
        text += '\n' + buildLabel(label, label.newId);
    });
    return text;
}
function makeFinder(type) {
    return (text)=>findAll(text, type).map(PROCESSORS[type]);
}
export function findMaxLabelId(textOrLabels) {
    if (textOrLabels != null && typeof textOrLabels.valueOf() === 'string') {
        textOrLabels = findLabels(textOrLabels);
    }
    const ids = textOrLabels.map((x)=>x.id).map((x)=>parseInt(x, 10)).filter((x)=>isFinite(x) && x > 0);
    return ids.length > 0 ? max(ids) || 0 : 0;
}
function findAll(text, type) {
    const flags = 'g' + (type === 'label' ? 'm' : '');
    const matches = [];
    const re = new RegExp(REGEXS[type].source, flags);
    let found = re.exec(text);
    while(found){
        matches.push(found);
        re.lastIndex = found.index + found[0].length;
        found = re.exec(text);
    }
    return matches;
}
function hasTitle(link) {
    return isObject(link) && link.title !== null && typeof link.title.valueOf() === 'string' && link.title.length > 0;
}
function buildLabel(link, id) {
    let markup = '[' + id + ']: ' + link.href;
    if (hasTitle(link)) {
        markup += ' "' + link.title + '"';
    }
    return markup;
}
function buildRef(link, id) {
    return '[' + link.text + '][' + id + ']';
}
